<?php
// $Id: autoadmin.util.inc,v 1.1 2009/09/18 20:44:36 rune Exp $

/**
 * @file
 * various utility functions
 *
 * @ingroup autoadmin
 * @author Rune Kaagaard rune@prescriba.com
 */

/**
 * Assembles page descriptions.
 *
 * @param array $schema
 * @return string
 */
function autoadmin_util_get_page_description($schema) {
  // Get descriptions.
  $description = autoadmin_util_set_filter_page_description(NULL, NULL);
  // If no descriptions.
  if (empty($description)) {
    return t('Showing all !title_plural', array('!title_plural' => $schema['title_plural']));
  }
  // Otherwise show filtering values and add a 'show all' link.
  return t('Showing !title_plural where !title is !value (!link)', array(
    '!title_plural' => $schema['title_plural'],
    '!title' => $description['title'],
    '!value' => $description['value'],
    '!link' => theme('autoadmin_operations', array(array(
      'title' => t('Show all !title_plural', array('!title_plural' => $schema['title_plural'])),
      'href' => $schema['path'] . '/list',
      'query' => 'show=all'
    ))),
  ));
}

/**
 * Adds to page description for a filter where filters applies
 * 
 * @param array $field
 * @param int $value
 */
function autoadmin_util_set_filter_page_description($field, $value) {
  static $description = array();
  if (empty($field)) {
    return $description;
  }
  $foreign_schema = autoadmin_schema_get_schema($field['relation']['foreign_table']);
  $description = array(
    'title' => 
      !empty($foreign_schema['title'])
      ? $foreign_schema['title']
      : $field['relation']['foreign_table']
    ,
    'value' => db_result(db_query(
      'SELECT %s FROM {%s} WHERE %s=%d',
      $field['relation']['foreign_alias'],
      $field['relation']['foreign_table'],
      $field['relation']['foreign_key'],
      $value
    ))
  );
}
/**
 * sets session value for a specific field in the schema
 *
 * @param array $schema
 * @param array $field
 * @param array $value
 */
function autoadmin_util_session_set_field_param($schema, $field, $value) {
  $_SESSION['autoadmin'][$schema['table']]['_fields'][$field['key']] = $value;
}

/**
 * gets session value for a specific field in the schema
 *
 * @param array $schema
 * @param array $field
 * @return mixed/NULL
 */
function autoadmin_util_session_get_field_param($schema, $field) {
  return
    !empty($_SESSION['autoadmin'][$schema['table']]['_fields'][$field['key']]) ?
    $_SESSION['autoadmin'][$schema['table']]['_fields'][$field['key']] :
    NULL
  ;
}

/**
 * if there is &show=all in url reset session data for schema
 *
 * @param array $schema
 */
function autoadmin_util_session_check_for_reset($schema) {
  if (!empty($_GET['show']) && $_GET['show'] == 'all') {
    unset($_SESSION['autoadmin'][$schema['table']]['_fields']);
  }
}

/**
 * clears all session data for schema
 *
 * @param array $schema
 */
function autoadmin_util_session_clear_schema($schema) {
  if (!empty($_SESSION['autoadmin'][$schema['table']])) {
    unset($_SESSION['autoadmin'][$schema['table']]);
  }
}

/**
 * Sets by ref default values for an int fields length and maxlength.
 * 
 * @global String $db_type
 * @param array $field
 * @param string $size
 * @param string $maxlength
 */
function autoadmin_util_field_lengths($field, &$size, &$maxlength) {
  global $db_type;
  if ($db_type == 'mysqli' || $db_type == 'mysql') {
    $mysql_length = (string)autoadmin_util_get_mysql_int_length($field);
    $size = !empty($field['size']) ? $field['size'] : $mysql_length;
    $maxlength = !empty($field['maxlength']) ? $field['maxlength'] : $mysql_length;
  } else {
    $size = !empty($field['size']) ? $field['size'] : '64';
    $maxlength = !empty($field['maxlength']) ? $field['maxlength'] : '128';
  }
}

/**
 * Gets maximum length for a mysql int field based on its size or if its unsigned or not.
 *
 * @see http://help.scibit.com/Mascon/masconMySQL_Field_Types.html
 * @param array $field
 * @return int
 *   Maximum number of characters.
 */
function autoadmin_util_get_mysql_int_length($field) {
  if (empty($field['size'])) {
    $field['size'] = NULL;
  }
  $add = empty($field['unsigned']) ? 0 : 1;
  switch ($field['size']) {
    case 'tiny':
      return (3 + $add);
    case 'small':
      return (5 + $add);
    case 'medium':
      return (8);
    case 'normal':
      return (10 + $add);
    case 'big':
      return (20);
    default:
      return (10 + $add);
  }
}

/**
 * returns name of callback function for field. Special rules apply to fields with relations.
 *
 * @param string $prefix
 * @param array $field
 * @return string
 */
function autoadmin_util_get_callback_function($prefix, $field) {
  if (empty($field['relation'])) {
    return $prefix . $field['autoadmin_type'];
  }
  else {
    return $prefix . $field['relation']['type'];
  }
}

/**
 * version of array_merge_recursive() where later arrays overwrites already existing values
 *
 * @credit taken from http://us2.php.net/array_merge_recursive by "michiel at synetic dot nl"
 * @return array
 */
function &autoadmin_util_array_merge_recursive_distinct() {
  $arrays = func_get_args();
  $array_merged = $arrays[0];
  for ($i = 1; $i < count($arrays); $i++) {
    if (is_array($arrays[$i])) {
      foreach ($arrays[$i] as $key => $val) {
        if (is_array($arrays[$i][$key])) {
          $array_merged[$key] = (!empty($array_merged[$key]) && is_array($array_merged[$key]))
						? autoadmin_util_array_merge_recursive_distinct($array_merged[$key], $arrays[$i][$key])
						: $arrays[$i][$key];
        }
        else {
          $array_merged[$key] = $val;
        }
      }
    }
  }
  return $array_merged;
}

/**
 * gets the primary value from a data row and schema
 *
 * @param array $row
 * @param array $schema
 * @return string/FALSE
 */
function autoadmin_util_get_pv($row, $schema) {
  return !empty($row[$schema['primary key'][0]]) ? $row[$schema['primary key'][0]] : FALSE;
}

/**
 * Checks whether a field has a filter value and returns it
 * 
 * @param array $schema
 * @param array $field
 * @return int
 */
function autoadmin_util_get_filter_value($schema, $field) {
  $value = NULL;
  if (!empty($_GET[$field['key']])) {
    $value = $_GET[$field['key']];
  }
  else {
    $value = autoadmin_util_session_get_field_param($schema, $field);
  }
  return $value;
}